@@ -1,5 +1,5 @@
 /* dymaxionmap --- Buckminster Fuller's unwrapped icosahedral globe.
- * Copyright (c) 2016-2018 Jamie Zawinski.
+ * Copyright © 2016-2025 Jamie Zawinski.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation for any purpose and without fee is hereby granted,
@@ -31,6 +31,8 @@
 
 #include "sphere.h"
 #include "normals.h"
+#include "easing.h"
+#include "doubletime.h"
 #include "texfont.h"
 #include "dymaxionmap-coords.h"
 
@@ -50,6 +52,16 @@
 #undef RANDSIGN
 #define RANDSIGN() ((random() & 1) ? 1 : -1)
 
+#undef BUILD_CACHE_PNG
+
+#ifdef BUILD_CACHE_PNG
+# ifdef HAVE_LIBPNG
+#  include <png.h>
+# else
+#  error BUILD_CACHE_PNG requires HAVE_LIBPNG
+# endif
+#endif /* BUILD_CACHE_PNG */
+
 static int do_roll;
 static int do_wander;
 static int do_texture;
@@ -146,21 +158,6 @@ typedef struct {
 static planetstruct *planets = NULL;
 
 
-static double
-double_time (void)
-{
-  struct timeval now;
-# ifdef GETTIMEOFDAY_TWO_ARGS
-  struct timezone tzp;
-  gettimeofday(&now, &tzp);
-# else
-  gettimeofday(&now);
-# endif
-
-  return (now.tv_sec + ((double) now.tv_usec * 0.000001));
-}
-
-
 /* Draw faint latitude and longitude lines into the RGBA XImage.
  */
 static void
@@ -301,6 +298,30 @@ create_daylight_mask (Display *dpy, Visual *v, int w, int h)
 }
 
 
+/* Reduce the XImage size by half.
+ */
+static XImage *
+shrink_image (Display *dpy, Visual *visual, XImage *image)
+{
+  int x, y;
+  int w = image->width;
+  int h = image->height;
+  XImage *out = XCreateImage (dpy, visual, image->depth, ZPixmap, 0, 0,
+                              w/2, h/2, 32, 0);
+  out->data = (char *) calloc (out->height, out->bytes_per_line);
+
+  /* We could do some blending or dithering here, but at these scales, and
+     with texture scaling happening anyway, I'm not sure we'd be able to
+     tell the difference.
+   */
+  for (y = 0; y < h; y++)
+    for (x = 0; x < w; x++)
+      XPutPixel (out, x >> 1, y >> 1, XGetPixel (image, x, y));
+  XDestroyImage (image);
+  return out;
+}
+
+
 static void
 load_images (ModeInfo *mi)
 {
@@ -325,17 +346,28 @@ load_images (ModeInfo *mi)
     {
       char *s = (i == 0 ? which_image : which_image2);
       XImage *image;
+      Bool builtin_p = False;
+
       if (!strcmp (s, "BUILTIN_DAY"))
-        image = image_data_to_ximage (MI_DISPLAY (mi), MI_VISUAL (mi),
-                                      earth_png, sizeof(earth_png));
+        {
+          image = image_data_to_ximage (MI_DISPLAY (mi), MI_VISUAL (mi),
+                                        earth_png, sizeof(earth_png));
+          builtin_p = True;
+        }
       else if (!strcmp (s, "BUILTIN_NIGHT"))
-        image = image_data_to_ximage (MI_DISPLAY (mi), MI_VISUAL (mi),
-                                      earth_night_png,sizeof(earth_night_png));
+        {
+          image = image_data_to_ximage (MI_DISPLAY (mi), MI_VISUAL (mi),
+                                     earth_night_png, sizeof(earth_night_png));
+          builtin_p = True;
+        }
       else if (!strcmp (s, "BUILTIN") ||
                !strcmp (s, "BUILTIN_FLAT") ||
                (i == 0 && !strcmp (s, "")))
-        image = image_data_to_ximage (MI_DISPLAY (mi), MI_VISUAL (mi),
-                                      earth_flat_png, sizeof(earth_flat_png));
+        {
+          image = image_data_to_ximage (MI_DISPLAY (mi), MI_VISUAL (mi),
+                                       earth_flat_png, sizeof(earth_flat_png));
+          builtin_p = True;
+        }
       else if (!strcmp (s, "NONE"))
         image = 0;
       else if (*s)
@@ -345,6 +377,11 @@ load_images (ModeInfo *mi)
 
       /* if (image) fprintf (stderr, "%s: %d: loaded %s\n", progname, i, s); */
 
+      /* The 2048x1024 images kill performance.  Turn this off in a few
+         years when Moore's Law has caught up again. */
+      while (builtin_p && image->width >= 2048)
+        image = shrink_image (MI_DISPLAY (mi), MI_VISUAL (mi), image);
+
       if (i == 0)
         gp->day = image;
       else
@@ -410,6 +447,11 @@ load_images (ModeInfo *mi)
      by half pixel to make sure we hit every pixel in 'out'. It would be
      cleaner to iterate over 'out' instead of over 'in' but
      dymaxionmap-coords.c only goes forward. This is... not super fast.
+
+     If the source image is 2048x1024, it takes ~7 seconds to compute this
+     4096x2034 'cvt' map on a 3.2 GHz iMac Pro.  I tried pre-computing it
+     and saving it as a PNG, but that PNG was an unreasonable 1.4 MB!
+     It's pretty cool looking, though: https://jwz.org/b/yjwx
    */
   {
     double W = 5.5;
@@ -444,13 +486,71 @@ load_images (ModeInfo *mi)
       }
   }
 
+# ifdef BUILD_CACHE_PNG
+  {
+    png_structp png_ptr;
+    png_infop info_ptr;
+    png_bytep row;
+    char *pngfile = "dymaxionmap_coords.png";
+    FILE *f = fopen (pngfile, "wb");
+    XImage *img = gp->cvt;
+    int x, y;
+    if (! f)
+      {
+        fprintf (stderr, "%s: unable to write %s\n", progname, pngfile);
+        exit (1);
+      }
+      
+    png_ptr = png_create_write_struct (PNG_LIBPNG_VER_STRING, 0, 0, 0);
+    if (!png_ptr) abort();
+    info_ptr = png_create_info_struct (png_ptr);
+    if (!info_ptr) abort();
+    if (setjmp (png_jmpbuf (png_ptr))) abort();
+
+    png_init_io (png_ptr, f);
+
+    png_set_IHDR (png_ptr, info_ptr, img->width, img->height, 8,
+                  PNG_COLOR_TYPE_RGBA, PNG_INTERLACE_NONE,
+                  PNG_COMPRESSION_TYPE_DEFAULT, PNG_FILTER_TYPE_DEFAULT);
+    png_write_info (png_ptr, info_ptr);
+
+    row = (png_bytep) malloc (4 * img->width * sizeof(png_byte));
+    if (!row) abort();
+    for (y = 0 ; y < img->height ; y++) {
+      for (x = 0 ; x < img->width ; x++) {
+        unsigned long p = XGetPixel (img, x, y);
+        row[x*4+3] = (p & 0xFF000000L) >> 24;
+        row[x*4+2] = (p & 0x00FF0000L) >> 16;
+        row[x*4+1] = (p & 0x0000FF00L) >> 8;
+        row[x*4+0] = (p & 0x000000FFL);
+      }
+      png_write_row (png_ptr, row);
+    }
+
+    png_write_end (png_ptr, 0);
+
+    png_free_data (png_ptr, info_ptr, PNG_FREE_ALL, -1);
+    png_destroy_write_struct (&png_ptr, 0);
+    free (row);
+
+    fclose (f);
+    fprintf (stderr, "%s: wrote %s\n", progname, pngfile);
+    exit (0);
+  }
+# endif /* BUILD_CACHE_PNG */
+
   /* A 128 GB iPhone 6s dies at around 540 frames, ~1 GB of XImages.
      A 16 GB iPad Air 2 dies at around 320 frames, ~640 MB.
      Caching on mobile doesn't matter much: we can just run at 100% CPU.
 
      On some systems it would be more efficient to cache the images inside
      a texture on the GPU instead of moving it from RAM to GPU every few
-     frames; but on other systems, we'd just run out of GPU memory instead. */
+     frames; but on other systems, we'd just run out of GPU memory instead.
+
+     The cache is all-or-nothing: caching every frame is useful, but
+     caching a subset is not.  If we have only cached, say, midnight to
+     1am, then that hour will move more quickly than the others.  Likewise,
+     caching every 4th frame would just give the stutter a backbeat. */
   {
     unsigned long cache_size = (gp->day->width * gp->day->height * 4 *
                                 gp->nimages);
@@ -508,13 +608,16 @@ cache_current_frame (ModeInfo *mi)
           uint32_t n = *night++;
           uint32_t x = i % w;
           uint32_t y = i / w;
+          /* This is W*H*8 float ops; can we do this with integer math?
+             This also might benefit from pthreads and vector ops, as
+             seen in "marbling", but that's a lot of work. */
           double r = dusk[y * w + ((x + xoff) % w)] / 256.0;
           double r2 = 1-r;
 # define ADD(M) (((unsigned long)             \
                   ((((d >> M) & 0xFF) * r) +  \
                    (((n >> M) & 0xFF) * r2))) \
                  << M)
-          /* #### Why is this ABGR instead of RGBA? */
+          /* Why is this ABGR instead of RGBA? */
           *out++ = (0xFF << 24) | ADD(16) | ADD(8) | ADD(0);
 # undef ADD
         }
@@ -539,7 +642,7 @@ cache_current_frame (ModeInfo *mi)
         unsigned long p  = day[(y>>1) * w + (x>>1)];
         unsigned long p2 = out[dy * w + dx];
         if (p2 & 0xFF000000)
-          /* RGBA nonzero alpha: initialized. Average with existing,
+          /* AGBR nonzero alpha: initialized. Average with existing,
              otherwise the grid lines look terrible. */
           p = (((((p>>24) & 0xFF) + ((p2>>24) & 0xFF)) >> 1) << 24 |
                ((((p>>16) & 0xFF) + ((p2>>16) & 0xFF)) >> 1) << 16 |
@@ -1378,25 +1481,6 @@ init_planet (ModeInfo * mi)
 }
 
 
-static GLfloat
-ease_fn (GLfloat r)
-{
-  return cos ((r/2 + 1) * M_PI) + 1; /* Smooth curve up, end at slope 1. */
-}
-
-
-static GLfloat
-ease_ratio (GLfloat r)
-{
-  GLfloat ease = 0.35;
-  if      (r <= 0)     return 0;
-  else if (r >= 1)     return 1;
-  else if (r <= ease)  return     ease * ease_fn (r / ease);
-  else if (r > 1-ease) return 1 - ease * ease_fn ((1 - r) / ease);
-  else                 return r;
-}
-
-
 ENTRYPOINT void
 draw_planet (ModeInfo * mi)
 {
@@ -1580,7 +1664,7 @@ draw_planet (ModeInfo * mi)
     case STEL_IN:  fold_ratio = 1; stel_ratio = gp->ratio; break;
     case STEL_OUT: fold_ratio = 1; stel_ratio = 1 - gp->ratio; break;
     case STARTUP:      /* Tilt in from flat */
-      glRotatef (-90 * ease_ratio (1 - gp->ratio), 1, 0, 0);
+      glRotatef (-90 * ease (EASE_IN_OUT_SINE, 1 - gp->ratio), 1, 0, 0);
       break;
 
     default: break;
@@ -1588,7 +1672,7 @@ draw_planet (ModeInfo * mi)
 
 # ifdef HAVE_MOBILE  /* Enlarge the icosahedron a bit to make it more visible */
     {
-      GLfloat s = 1 + 1.3 * ease_ratio (fold_ratio);
+      GLfloat s = 1 + 1.3 * ease (EASE_IN_OUT_SINE, fold_ratio);
       glScalef (s, s, s);
     }
 # endif
@@ -1596,11 +1680,13 @@ draw_planet (ModeInfo * mi)
     if (gp->state == SPIN)
       {
         align_axis (mi, 0);
-        glRotatef (ease_ratio (gp->ratio) * 360 * 3, 0, 0, 1);
+        glRotatef (ease (EASE_IN_OUT_SINE, gp->ratio) * 360 * 3, 0, 0, 1);
         align_axis (mi, 1);
       }
 
-    draw_triangles (mi, ease_ratio (fold_ratio), ease_ratio (stel_ratio));
+    draw_triangles (mi,
+                    ease (EASE_IN_OUT_SINE, fold_ratio),
+                    ease (EASE_IN_OUT_SINE, stel_ratio));
 
     if (gp->state == AXIS)
       draw_axis(mi);
