@@ -5,7 +5,7 @@
 static const char sccsid[] = "@(#)projectiveplane.c  1.1 14/01/03 xlockmore";
 #endif
 
-/* Copyright (c) 2013-2021 Carsten Steger <carsten@mirsanmir.org>. */
+/* Copyright (c) 2013-2026 Carsten Steger <carsten@mirsanmir.org>. */
 
 /*
  * Permission to use, copy, modify, and distribute this software and its
@@ -27,6 +27,7 @@ static const char sccsid[] = "@(#)projectiveplane.c  1.1 14/01/03 xlockmore";
  * C. Steger - 20/12/05: Added per-fragment shading
  * C. Steger - 20/12/06: Moved all GLSL support code into glsl-utils.[hc]
  * C. Steger - 20/12/30: Make the shader code work under macOS and iOS
+ * C. Steger - 26/01/01: Make the code work in an OpenGL core profile
  */
 
 /*
@@ -414,7 +415,7 @@ typedef struct {
 #ifdef HAVE_GLSL
   GLfloat uv[(NUMU+1)*(NUMV+1)][2];
   GLuint indices[4*(NUMU+1)*(NUMV+1)];
-  Bool use_shaders, buffers_initialized;
+  Bool use_shaders, buffers_initialized, use_vao;
   GLuint shader_program;
   GLint vertex_uv_index, vertex_t_index, color_index;
   GLint mat_rot_index, mat_p_index, bool_persp_index;
@@ -429,6 +430,7 @@ typedef struct {
   GLint texture_sampler_index;
   GLuint vertex_uv_buffer, vertex_t_buffer;
   GLuint color_buffer, indices_buffer;
+  GLuint vertex_array_object;
   GLint ni, ne, nt;
 #endif /* HAVE_GLSL */
 } projectiveplanestruct;
@@ -438,189 +440,181 @@ static projectiveplanestruct *projectiveplane = (projectiveplanestruct *) NULL;
 
 #ifdef HAVE_GLSL
 
-/* The GLSL versions that correspond to different versions of OpenGL. */
-static const GLchar *shader_version_2_1 =
-  "#version 120\n";
-static const GLchar *shader_version_3_0 =
-  "#version 130\n";
-static const GLchar *shader_version_3_0_es =
-  "#version 300 es\n"
-  "precision highp float;\n"
-  "precision highp int;\n";
+/* The vertex shader code. */
+static const GLchar *vertex_shader = "\
+#ifdef GL_ES                                                             \n\
+precision highp float;                                                   \n\
+precision highp int;                                                     \n\
+precision highp sampler2D;                                               \n\
+#endif                                                                   \n\
+                                                                         \n\
+#if __VERSION__ <= 120                                                   \n\
+attribute vec2 VertexUV;                                                 \n\
+attribute vec4 VertexT;                                                  \n\
+attribute vec4 VertexColor;                                              \n\
+varying vec3 Normal;                                                     \n\
+varying vec4 Color;                                                      \n\
+varying vec4 TexCoord;                                                   \n\
+#else                                                                    \n\
+in vec2 VertexUV;                                                        \n\
+in vec4 VertexT;                                                         \n\
+in vec4 VertexColor;                                                     \n\
+out vec3 Normal;                                                         \n\
+out vec4 Color;                                                          \n\
+out vec4 TexCoord;                                                       \n\
+#endif                                                                   \n\
+                                                                         \n\
+uniform mat4 MatRot4D;                                                   \n\
+uniform mat4 MatProj;                                                    \n\
+uniform bool BoolPersp;                                                  \n\
+uniform vec4 Offset4D;                                                   \n\
+uniform vec4 Offset3D;                                                   \n\
+uniform bool BoolTextures;                                               \n\
+                                                                         \n\
+void main(void)                                                          \n\
+{                                                                        \n\
+  const float EPSILON = 1.0e-7f;                                         \n\
+  float u, v, su, cu, s2u, c2u, sv2, cv2, sv4, cv4;                      \n\
+  vec3 p, pu, pv;                                                        \n\
+  u = VertexUV.x;                                                        \n\
+  v = VertexUV.y;                                                        \n\
+  su = sin(u);                                                           \n\
+  cu = cos(u);                                                           \n\
+  s2u = sin(2.0f*u);                                                     \n\
+  c2u = cos(2.0f*u);                                                     \n\
+  sv2 = sin(0.5f*v);                                                     \n\
+  cv2 = cos(0.5f*v);                                                     \n\
+  sv4 = sin(0.25f*v);                                                    \n\
+  cv4 = cos(0.25f*v);                                                    \n\
+  vec4 xx = vec4(0.5f*s2u*sv4*sv4,                                       \n\
+                 0.5f*su*sv2,                                            \n\
+                 0.5f*cu*sv2,                                            \n\
+                 0.5f*(su*su*sv4*sv4-cv4*cv4));                          \n\
+  if (v < EPSILON)                                                       \n\
+  {                                                                      \n\
+    v = EPSILON;                                                         \n\
+    sv2 = sin(0.5f*v);                                                   \n\
+    cv2 = cos(0.5f*v);                                                   \n\
+    sv4 = sin(0.25f*v);                                                  \n\
+  }                                                                      \n\
+  vec4 xxu = vec4(c2u*sv4*sv4,                                           \n\
+                  0.5f*cu*sv2,                                           \n\
+                  -0.5f*su*sv2,                                          \n\
+                  0.5f*s2u*sv4*sv4);                                     \n\
+  vec4 xxv = vec4(0.125f*s2u*sv2,                                        \n\
+                  0.25f*su*cv2,                                          \n\
+                  0.25f*cu*cv2,                                          \n\
+                  0.125f*(su*su+1.0f)*sv2);                              \n\
+  vec4 x = MatRot4D*xx+Offset4D;                                         \n\
+  vec4 xu = MatRot4D*xxu;                                                \n\
+  vec4 xv = MatRot4D*xxv;                                                \n\
+  if (BoolPersp)                                                         \n\
+  {                                                                      \n\
+    vec3 r = x.xyz;                                                      \n\
+    float s = x.w;                                                       \n\
+    float t = s*s;                                                       \n\
+    p = r/s+Offset3D.xyz;                                                \n\
+    pu = (s*xu.xyz-r*xu.w)/t;                                            \n\
+    pv = (s*xv.xyz-r*xv.w)/t;                                            \n\
+  }                                                                      \n\
+  else                                                                   \n\
+  {                                                                      \n\
+    p = x.xyz+Offset3D.xyz;                                              \n\
+    pu = xu.xyz;                                                         \n\
+    pv = xv.xyz;                                                         \n\
+  }                                                                      \n\
+  vec4 Position = vec4(p,1.0);                                           \n\
+  Normal = normalize(cross(pu,pv));                                      \n\
+  gl_Position = MatProj*Position;                                        \n\
+  Color = VertexColor;                                                   \n\
+  if (BoolTextures)                                                      \n\
+    TexCoord = VertexT;                                                  \n\
+}                                                                        \n";
 
-/* The vertex shader code is composed of code fragments that depend on
-   the OpenGL version and code fragments that are version-independent.
-   They are concatenated by glsl_CompileAndLinkShaders in the function
-   init_glsl(). */
-static const GLchar *vertex_shader_attribs_2_1 =
-  "attribute vec2 VertexUV;\n"
-  "attribute vec4 VertexT;\n"
-  "attribute vec4 VertexColor;\n"
-  "\n"
-  "varying vec3 Normal;\n"
-  "varying vec4 Color;\n"
-  "varying vec4 TexCoord;\n"
-  "\n";
-static const GLchar *vertex_shader_attribs_3_0 =
-  "in vec2 VertexUV;\n"
-  "in vec4 VertexT;\n"
-  "in vec4 VertexColor;\n"
-  "\n"
-  "out vec3 Normal;\n"
-  "out vec4 Color;\n"
-  "out vec4 TexCoord;\n"
-  "\n";
-static const GLchar *vertex_shader_main =
-  __extension__
-  "uniform mat4 MatRot4D;\n"
-  "uniform mat4 MatProj;\n"
-  "uniform bool BoolPersp;\n"
-  "uniform vec4 Offset4D;\n"
-  "uniform vec4 Offset3D;\n"
-  "uniform bool BoolTextures;\n"
-  "\n"
-  "void main (void)\n"
-  "{\n"
-  "  const float EPSILON = 1.0e-7f;\n"
-  "  float u, v, su, cu, s2u, c2u, sv2, cv2, sv4, cv4;\n"
-  "  vec3 p, pu, pv;\n"
-  "  u = VertexUV.x;\n"
-  "  v = VertexUV.y;\n"
-  "  su = sin(u)\n;"
-  "  cu = cos(u)\n;"
-  "  s2u = sin(2.0f*u)\n;"
-  "  c2u = cos(2.0f*u)\n;"
-  "  sv2 = sin(0.5f*v)\n;"
-  "  cv2 = cos(0.5f*v)\n;"
-  "  sv4 = sin(0.25f*v)\n;"
-  "  cv4 = cos(0.25f*v)\n;"
-  "  vec4 xx = vec4(0.5f*s2u*sv4*sv4,\n"
-  "                 0.5f*su*sv2,\n"
-  "                 0.5f*cu*sv2,\n"
-  "                 0.5f*(su*su*sv4*sv4-cv4*cv4));\n"
-  "  if (v < EPSILON)\n"
-  "  {\n"
-  "    v = EPSILON;\n"
-  "    sv2 = sin(0.5f*v)\n;"
-  "    cv2 = cos(0.5f*v)\n;"
-  "    sv4 = sin(0.25f*v)\n;"
-  "  }\n"
-  "  vec4 xxu = vec4(c2u*sv4*sv4,\n"
-  "                  0.5f*cu*sv2,\n"
-  "                  -0.5f*su*sv2,\n"
-  "                  0.5f*s2u*sv4*sv4);\n"
-  "  vec4 xxv = vec4(0.125f*s2u*sv2,\n"
-  "                  0.25f*su*cv2,\n"
-  "                  0.25f*cu*cv2,\n"
-  "                  0.125f*(su*su+1.0f)*sv2);\n"
-  "  vec4 x = MatRot4D*xx+Offset4D;\n"
-  "  vec4 xu = MatRot4D*xxu;\n"
-  "  vec4 xv = MatRot4D*xxv;\n"
-  "  if (BoolPersp)\n"
-  "  {\n"
-  "    vec3 r = x.xyz;\n"
-  "    float s = x.w;\n"
-  "    float t = s*s;\n"
-  "    p = r/s+Offset3D.xyz;\n"
-  "    pu = (s*xu.xyz-r*xu.w)/t;\n"
-  "    pv = (s*xv.xyz-r*xv.w)/t;\n"
-  "  }\n"
-  "  else\n"
-  "  {\n"
-  "    p = x.xyz+Offset3D.xyz;\n"
-  "    pu = xu.xyz;\n"
-  "    pv = xv.xyz;\n"
-  "  }\n"
-  "  vec4 Position = vec4(p,1.0);\n"
-  "  Normal = normalize(cross(pu,pv));\n"
-  "  gl_Position = MatProj*Position;\n"
-  "  Color = VertexColor;\n"
-  "  if (BoolTextures)\n"
-  "    TexCoord = VertexT;\n"
-  "}\n";
-
-/* The fragment shader code is composed of code fragments that depend on
-   the OpenGL version and code fragments that are version-independent.
-   They are concatenated by glsl_CompileAndLinkShaders in the function
-   init_glsl(). */
-static const GLchar *fragment_shader_attribs_2_1 =
-  "varying vec3 Normal;\n"
-  "varying vec4 Color;\n"
-  "varying vec4 TexCoord;\n"
-  "\n";
-static const GLchar *fragment_shader_attribs_3_0 =
-  "in vec3 Normal;\n"
-  "in vec4 Color;\n"
-  "in vec4 TexCoord;\n"
-  "\n"
-  "out vec4 FragColor;\n"
-  "\n";
-static const GLchar *fragment_shader_main =
-  __extension__
-  "uniform bool DrawLines;\n"
-  "uniform vec4 LtGlblAmbient;\n"
-  "uniform vec4 LtAmbient, LtDiffuse, LtSpecular;\n"
-  "uniform vec3 LtDirection, LtHalfVector;\n"
-  "uniform vec4 MatFrontAmbient, MatBackAmbient;\n"
-  "uniform vec4 MatFrontDiffuse, MatBackDiffuse;\n"
-  "uniform vec4 MatSpecular;\n"
-  "uniform float MatShininess;\n"
-  "uniform bool BoolTextures;\n"
-  "uniform sampler2D TextureSampler;"
-  "\n"
-  "void main (void)\n"
-  "{\n"
-  "  vec4 color;\n"
-  "  if (DrawLines)\n"
-  "  {\n"
-  "    color = Color;\n"
-  "  }\n"
-  "  else\n"
-  "  {\n"
-  "    vec3 normalDirection;\n"
-  "    vec4 ambientColor, diffuseColor, sceneColor;\n"
-  "    vec4 ambientLighting, diffuseReflection, specularReflection;\n"
-  "    float ndotl, ndoth, pf;\n"
-  "    \n"
-  "    if (gl_FrontFacing)\n"
-  "    {\n"
-  "      normalDirection = normalize(Normal);\n"
-  "      sceneColor = Color*MatFrontAmbient*LtGlblAmbient;\n"
-  "      ambientColor = Color*MatFrontAmbient;\n"
-  "      diffuseColor = Color*MatFrontDiffuse;\n"
-  "    }\n"
-  "    else\n"
-  "    {\n"
-  "      normalDirection = -normalize(Normal);\n"
-  "      sceneColor = Color*MatBackAmbient*LtGlblAmbient;\n"
-  "      ambientColor = Color*MatBackAmbient;\n"
-  "      diffuseColor = Color*MatBackDiffuse;\n"
-  "    }\n"
-  "    \n"
-  "    ndotl = max(0.0,dot(normalDirection,LtDirection));\n"
-  "    ndoth = max(0.0,dot(normalDirection,LtHalfVector));\n"
-  "    if (ndotl == 0.0)\n"
-  "      pf = 0.0;\n"
-  "    else\n"
-  "      pf = pow(ndoth,MatShininess);\n"
-  "    ambientLighting = ambientColor*LtAmbient;\n"
-  "    diffuseReflection = LtDiffuse*diffuseColor*ndotl;\n"
-  "    specularReflection = LtSpecular*MatSpecular*pf;\n"
-  "    color = sceneColor+ambientLighting+diffuseReflection;\n";
-static const GLchar *fragment_shader_out_2_1 =
-  "    if (BoolTextures)\n"
-  "      color *= texture2D(TextureSampler,TexCoord.st);"
-  "    color += specularReflection;\n"
-  "  }\n"
-  "  gl_FragColor = clamp(color,0.0,1.0);\n"
-  "}\n";
-static const GLchar *fragment_shader_out_3_0 =
-  "    if (BoolTextures)\n"
-  "      color *= texture(TextureSampler,TexCoord.st);"
-  "    color += specularReflection;\n"
-  "  }\n"
-  "  FragColor = clamp(color,0.0,1.0);\n"
-  "}\n";
+/* The fragment shader code. */
+static const GLchar *fragment_shader = "\
+#ifdef GL_ES                                                             \n\
+precision highp float;                                                   \n\
+precision highp int;                                                     \n\
+precision highp sampler2D;                                               \n\
+#endif                                                                   \n\
+                                                                         \n\
+#if __VERSION__ <= 120                                                   \n\
+varying vec3 Normal;                                                     \n\
+varying vec4 Color;                                                      \n\
+varying vec4 TexCoord;                                                   \n\
+#else                                                                    \n\
+in vec3 Normal;                                                          \n\
+in vec4 Color;                                                           \n\
+in vec4 TexCoord;                                                        \n\
+out vec4 FragColor;                                                      \n\
+#endif                                                                   \n\
+                                                                         \n\
+uniform bool DrawLines;                                                  \n\
+uniform vec4 LtGlblAmbient;                                              \n\
+uniform vec4 LtAmbient, LtDiffuse, LtSpecular;                           \n\
+uniform vec3 LtDirection, LtHalfVector;                                  \n\
+uniform vec4 MatFrontAmbient, MatBackAmbient;                            \n\
+uniform vec4 MatFrontDiffuse, MatBackDiffuse;                            \n\
+uniform vec4 MatSpecular;                                                \n\
+uniform float MatShininess;                                              \n\
+uniform bool BoolTextures;                                               \n\
+uniform sampler2D TextureSampler;                                        \n\
+                                                                         \n\
+void main(void)                                                          \n\
+{                                                                        \n\
+  vec4 color;                                                            \n\
+  if (DrawLines)                                                         \n\
+  {                                                                      \n\
+    color = Color;                                                       \n\
+  }                                                                      \n\
+  else                                                                   \n\
+  {                                                                      \n\
+    vec3 normalDirection;                                                \n\
+    vec4 ambientColor, diffuseColor, sceneColor;                         \n\
+    vec4 ambientLighting, diffuseReflection, specularReflection;         \n\
+    float ndotl, ndoth, pf;                                              \n\
+                                                                         \n\
+    if (gl_FrontFacing)                                                  \n\
+    {                                                                    \n\
+      normalDirection = normalize(Normal);                               \n\
+      sceneColor = Color*MatFrontAmbient*LtGlblAmbient;                  \n\
+      ambientColor = Color*MatFrontAmbient;                              \n\
+      diffuseColor = Color*MatFrontDiffuse;                              \n\
+    }                                                                    \n\
+    else                                                                 \n\
+    {                                                                    \n\
+      normalDirection = -normalize(Normal);                              \n\
+      sceneColor = Color*MatBackAmbient*LtGlblAmbient;                   \n\
+      ambientColor = Color*MatBackAmbient;                               \n\
+      diffuseColor = Color*MatBackDiffuse;                               \n\
+    }                                                                    \n\
+                                                                         \n\
+    ndotl = max(0.0,dot(normalDirection,LtDirection));                   \n\
+    ndoth = max(0.0,dot(normalDirection,LtHalfVector));                  \n\
+    if (ndotl == 0.0)                                                    \n\
+      pf = 0.0;                                                          \n\
+    else                                                                 \n\
+      pf = pow(ndoth,MatShininess);                                      \n\
+    ambientLighting = ambientColor*LtAmbient;                            \n\
+    diffuseReflection = LtDiffuse*diffuseColor*ndotl;                    \n\
+    specularReflection = LtSpecular*MatSpecular*pf;                      \n\
+    color = sceneColor+ambientLighting+diffuseReflection;                \n\
+    if (BoolTextures)                                                    \n\
+#if __VERSION__ <= 120                                                   \n\
+      color *= texture2D(TextureSampler,TexCoord.st).rrra;               \n\
+#else                                                                    \n\
+      color *= texture(TextureSampler,TexCoord.st).rrra;                 \n\
+#endif                                                                   \n\
+    color += specularReflection;                                         \n\
+    color.a = diffuseColor.a;                                            \n\
+  }                                                                      \n\
+#if __VERSION__ <= 120                                                   \n\
+  gl_FragColor = clamp(color,0.0,1.0);                                   \n\
+#else                                                                    \n\
+  FragColor = clamp(color,0.0,1.0);                                      \n\
+#endif                                                                   \n\
+}                                                                        \n";
 
 #endif /* HAVE_GLSL */
 
@@ -1280,6 +1274,7 @@ static int projective_plane_ff(ModeInfo *mi, double umin, double umax,
   if (pp->marks)
   {
     glEnable(GL_TEXTURE_2D);
+    glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
 #ifndef HAVE_JWZGLES
     glLightModeli(GL_LIGHT_MODEL_COLOR_CONTROL,GL_SEPARATE_SPECULAR_COLOR);
 #endif
@@ -1936,6 +1931,9 @@ static int projective_plane_pf(ModeInfo *mi, double umin, double umax,
     }
   }
 
+  if (pp->use_vao)
+    glBindVertexArray(pp->vertex_array_object);
+
   glActiveTexture(GL_TEXTURE0);
   glBindTexture(GL_TEXTURE_2D,pp->tex_name);
   glUniform1i(pp->texture_sampler_index,0);
@@ -1985,6 +1983,9 @@ static int projective_plane_pf(ModeInfo *mi, double umin, double umax,
   glBindBuffer(GL_ARRAY_BUFFER,0);
   glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,0);
 
+  if (pp->use_vao)
+    glBindVertexArray(0);
+
   glUseProgram(0);
 
   polys = 2*NUMU*NUMV;
@@ -2008,9 +2009,14 @@ static void gen_texture(ModeInfo *mi)
   glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
   glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
   glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
-  glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
-  glTexImage2D(GL_TEXTURE_2D,0,GL_LUMINANCE,TEX_DIMENSION,TEX_DIMENSION,0,
-               GL_LUMINANCE,GL_UNSIGNED_BYTE,texture);
+#ifdef HAVE_GLSL
+  if (pp->use_shaders)
+    glTexImage2D(GL_TEXTURE_2D,0,GL_R8,TEX_DIMENSION,TEX_DIMENSION,0,
+                 GL_RED,GL_UNSIGNED_BYTE,texture);
+  else
+#endif /* HAVE_GLSL */
+    glTexImage2D(GL_TEXTURE_2D,0,GL_LUMINANCE,TEX_DIMENSION,TEX_DIMENSION,0,
+                 GL_LUMINANCE,GL_UNSIGNED_BYTE,texture);
 }
 
 
@@ -2021,8 +2027,8 @@ static void init_glsl(ModeInfo *mi)
   projectiveplanestruct *pp = &projectiveplane[MI_SCREEN(mi)];
   GLint gl_major, gl_minor, glsl_major, glsl_minor;
   GLboolean gl_gles3;
-  const GLchar *vertex_shader_source[3];
-  const GLchar *fragment_shader_source[4];
+  const GLchar *vertex_shader_source[2];
+  const GLchar *fragment_shader_source[2];
 
   /* Determine whether to use shaders to render the projective plane. */
   pp->use_shaders = False;
@@ -2031,54 +2037,18 @@ static void init_glsl(ModeInfo *mi)
   pp->ni = 0;
   pp->ne = 0;
   pp->nt = 0;
+  pp->use_vao = False;
 
   if (!glsl_GetGlAndGlslVersions(&gl_major,&gl_minor,&glsl_major,&glsl_minor,
                                  &gl_gles3))
     return;
-  if (!gl_gles3)
-  {
-    if (gl_major < 3 ||
-        (glsl_major < 1 || (glsl_major == 1 && glsl_minor < 30)))
-    {
-      if ((gl_major < 2 || (gl_major == 2 && gl_minor < 1)) ||
-          (glsl_major < 1 || (glsl_major == 1 && glsl_minor < 20)))
-        return;
-      /* We have at least OpenGL 2.1 and at least GLSL 1.20. */
-      vertex_shader_source[0] = shader_version_2_1;
-      vertex_shader_source[1] = vertex_shader_attribs_2_1;
-      vertex_shader_source[2] = vertex_shader_main;
-      fragment_shader_source[0] = shader_version_2_1;
-      fragment_shader_source[1] = fragment_shader_attribs_2_1;
-      fragment_shader_source[2] = fragment_shader_main;
-      fragment_shader_source[3] = fragment_shader_out_2_1;
-    }
-    else
-    {
-      /* We have at least OpenGL 3.0 and at least GLSL 1.30. */
-      vertex_shader_source[0] = shader_version_3_0;
-      vertex_shader_source[1] = vertex_shader_attribs_3_0;
-      vertex_shader_source[2] = vertex_shader_main;
-      fragment_shader_source[0] = shader_version_3_0;
-      fragment_shader_source[1] = fragment_shader_attribs_3_0;
-      fragment_shader_source[2] = fragment_shader_main;
-      fragment_shader_source[3] = fragment_shader_out_3_0;
-    }
-  }
-  else /* gl_gles3 */
-  {
-    if (gl_major < 3 || glsl_major < 3)
-      return;
-    /* We have at least OpenGL ES 3.0 and at least GLSL ES 3.0. */
-    vertex_shader_source[0] = shader_version_3_0_es;
-    vertex_shader_source[1] = vertex_shader_attribs_3_0;
-    vertex_shader_source[2] = vertex_shader_main;
-    fragment_shader_source[0] = shader_version_3_0_es;
-    fragment_shader_source[1] = fragment_shader_attribs_3_0;
-    fragment_shader_source[2] = fragment_shader_main;
-    fragment_shader_source[3] = fragment_shader_out_3_0;
-  }
-  if (!glsl_CompileAndLinkShaders(3,vertex_shader_source,
-                                  4,fragment_shader_source,
+
+  vertex_shader_source[0] = glsl_GetGLSLVersionString();
+  vertex_shader_source[1] = vertex_shader;
+  fragment_shader_source[0] = glsl_GetGLSLVersionString();
+  fragment_shader_source[1] = fragment_shader;
+  if (!glsl_CompileAndLinkShaders(2,vertex_shader_source,
+                                  2,fragment_shader_source,
                                   &pp->shader_program))
     return;
   pp->vertex_uv_index = glGetAttribLocation(pp->shader_program,"VertexUV");
@@ -2150,6 +2120,10 @@ static void init_glsl(ModeInfo *mi)
   glGenBuffers(1,&pp->color_buffer);
   glGenBuffers(1,&pp->indices_buffer);
 
+  pp->use_vao = glsl_IsCoreProfile();
+  if (pp->use_vao)
+    glGenVertexArrays(1,&pp->vertex_array_object);
+
   pp->use_shaders = True;
 }
 
@@ -2204,13 +2178,13 @@ static void init(ModeInfo *mi)
   pp->offset3d[2] = -1.2;
   pp->offset3d[3] = 0.0;
 
-  gen_texture(mi);
-  setup_projective_plane(mi,0.0,2.0*M_PI,0.0,2.0*M_PI);
-
 #ifdef HAVE_GLSL
   init_glsl(mi);
 #endif /* HAVE_GLSL */
 
+  gen_texture(mi);
+  setup_projective_plane(mi,0.0,2.0*M_PI,0.0,2.0*M_PI);
+
 #ifdef HAVE_ANDROID
   /* glPolygonMode(...,GL_LINE) is not supported for an OpenGL ES 1.1
      context. */
@@ -2651,6 +2625,8 @@ ENTRYPOINT void free_projectiveplane(ModeInfo *mi)
     glDeleteBuffers(1,&pp->vertex_t_buffer);
     glDeleteBuffers(1,&pp->color_buffer);
     glDeleteBuffers(1,&pp->indices_buffer);
+    if (pp->use_vao)
+      glDeleteVertexArrays(1,&pp->vertex_array_object);
     if (pp->shader_program != 0)
     {
       glUseProgram(0);
